@@ -19,6 +19,14 @@ from character import Character
 from animatorystates import *
 from location import Location
 
+# Minimum distance away Eric must be to be considered nearby (used to decide
+# whether the frog should hop)
+ERIC_PROXIMITY = 3
+
+# Coordinates of the frog within its SIT sprite (used for placement in a cup
+# and detecting collision with a character's head)
+FROG_X, FROG_Y = 1, 3
+
 class Animal(Character):
     def __init__(self, animal_id, tap_id, initial_as, location):
         Character.__init__(self, animal_id, animal_id)
@@ -52,7 +60,7 @@ class Animal(Character):
 
     def is_eric_nearby(self):
         eric_x, eric_y = self.get_location_of_eric()
-        return self.y == eric_y and abs(self.x - eric_x) <= 3
+        return self.y == eric_y and abs(self.x - eric_x) <= ERIC_PROXIMITY
 
 class Mouse(Animal):
     def __init__(self, mouse_id, tap_id, initial_as, location, immortal=True):
@@ -86,7 +94,7 @@ class Frog(Animal):
     def insert_into_cup(self, cup):
         self.trapped = True
         self.animatory_state = SIT
-        self.x, self.y = cup.x - 1, cup.y - 3
+        self.x, self.y = cup.x - FROG_X, cup.y - FROG_Y
 
     def fall_from_cup(self, cup):
         cup.remove_frog(self)
