@@ -20,6 +20,8 @@ from animatorystates import *
 from location import Location
 
 SPEED = 3
+PEDAL_MOMENTUM = 2
+MAX_MOMENTUM = 50
 
 class Bike(Character):
     def __init__(self, bike_id, tap_id, initial_as, unchain_location):
@@ -45,7 +47,7 @@ class Bike(Character):
         self.hide()
 
     def pedal(self):
-        self.momentum = min(self.momentum + 2, 50)
+        self.momentum = min(self.momentum + PEDAL_MOMENTUM, MAX_MOMENTUM)
 
     def start_wheeling(self, rider):
         self.walk_delay = SPEED
