@@ -17,18 +17,58 @@
 
 import random
 import pygame
-from character import Character, DeskLid
-from animal import Mouse, Frog
+import character as char
+from desklid import DeskLid
+import animal
 from bike import Bike
+import droppable
 from plant import Plant
 from eric import Eric
 from pellet import Pellet
 from water import Water
 from stinkbomb import Stinkbomb
 from ai import CommandListTemplate
-from skool import Location
+from location import Location
 from animatorystates import *
 
+# The transparent colour in sprites.png
+SPRITE_COLORKEY = (0, 254, 0)
+
+# Size of the sprite matrix in sprites.png
+SPRITE_ROWS = 8
+SPRITE_COLUMNS = 16
+
+# Distance in front of Eric a character must be in order to be kissable
+KISS_DISTANCE = 2
+
+# Maximum distances at which a character is considered nearby (for lines-giving
+# purposes)
+NEARBY_X_RANGE = 10
+NEARBY_Y_RANGE = 3
+
+# Maximum distance from Eric at which a character is considered beside him
+BESIDE_ERIC_X_RANGE = 4
+
+# Locations at which a new immortal mouse may be released (after Eric catches
+# one)
+MOUSE_LOCATIONS = (
+    (10, 3),  (88, 3),  (170, 3),
+    (10, 10), (88, 10), (170, 10),
+    (10, 17), (88, 17), (170, 17)
+)
+# ID of the command list used by mortal mice
+MOUSE_COMMAND_LIST = 'Mouse'
+# ID of the sprite group used by mortal mice
+MOUSE_SPRITE_GROUP = 'MOUSE'
+
+# Marker replaced by a teacher's title in EINSTEIN's speech
+TITLE_MARKER = '$TITLE'
+# Marker replaced by the grassee's name in EINSTEIN's speech
+GRASSEE_MARKER = '$1'
+
+# Character that will be replaced by a newline in lines messages
+LINES_MESSAGE_NEWLINE = '^'
+
 class Cast:
     def __init__(self, sprites):
         self.sprites = sprites
@@ -78,17 +118,16 @@ class Cast:
     def add_character(self, character_id, name_and_title, sprite_group_id, initial_as, flags):
         names = name_and_title.partition('/')
         name, title = names[0], names[2]
-        character = Character(character_id, name, flags)
-        character.title = title
-        character.set_animatory_states(*self.sprite_groups[sprite_group_id])
-        character.initialise_animatory_state(initial_as)
-        self.everything.append(character)
-        self.characters[character_id] = character
-        self.character_list.append(character)
-        self.movables.append(character)
+        c = char.Character(character_id, name, flags)
+        c.title = title
+        c.set_animatory_states(*self.sprite_groups[sprite_group_id])
+        c.initialise_animatory_state(initial_as)
+        self.everything.append(c)
+        self.characters[character_id] = c
+        self.character_list.append(c)
+        self.movables.append(c)
 
-    def add_pellet(self, character_id, sprite_group_id, tap_id, pellet_range, hit_zone):
-        pellet_id = character_id + '-PELLET'
+    def add_pellet(self, character_id, pellet_id, sprite_group_id, tap_id, pellet_range, hit_zone):
         pellet = Pellet(pellet_id, tap_id, pellet_range, hit_zone)
         pellet.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(pellet)
@@ -96,47 +135,45 @@ class Cast:
         self.get(character_id).pellet = pellet
 
     def add_water_drop(self, object_id, sprite_group_id, tap_id):
-        self.water_drop = WaterDrop(object_id, tap_id)
+        self.water_drop = droppable.WaterDrop(object_id, tap_id)
         self.water_drop.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(self.water_drop)
         self.movables.append(self.water_drop)
 
     def add_sherry_drop(self, object_id, sprite_group_id, tap_id):
-        self.sherry_drop = SherryDrop(object_id, tap_id)
+        self.sherry_drop = droppable.SherryDrop(object_id, tap_id)
         self.sherry_drop.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(self.sherry_drop)
         self.movables.append(self.sherry_drop)
 
     def add_conker(self, object_id, sprite_group_id, tap_id, min_x, max_x, min_y, max_y):
-        self.conker = Conker(object_id, tap_id, min_x, max_x, min_y, max_y)
+        self.conker = droppable.Conker(object_id, tap_id, min_x, max_x, min_y, max_y)
         self.conker.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(self.conker)
         self.movables.append(self.conker)
 
-    def add_water(self, character_id, sprite_group_id, tap_id):
-        water_id = character_id + '-WATER'
+    def add_water(self, character_id, water_id, sprite_group_id, tap_id):
         water = Water(water_id, tap_id)
         water.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(water)
         self.movables.append(water)
         self.get(character_id).water = water
 
-    def add_stinkbomb(self, character_id, sprite_group_id, tap_id):
-        stinkbomb_id = character_id + '-STINKBOMB'
+    def add_stinkbomb(self, character_id, stinkbomb_id, sprite_group_id, tap_id):
         stinkbomb = Stinkbomb(stinkbomb_id, tap_id)
         stinkbomb.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(stinkbomb)
         self.movables.append(stinkbomb)
         self.get(character_id).stinkbomb = stinkbomb
 
-    def add_desk_lid(self, sprite_group_id, tap_id):
-        self.desk_lid = DeskLid('DeskLid', tap_id)
+    def add_desk_lid(self, desk_lid_id, sprite_group_id, tap_id):
+        self.desk_lid = DeskLid(desk_lid_id, tap_id)
         self.desk_lid.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(self.desk_lid)
         self.movables.append(self.desk_lid)
 
     def add_mouse(self, mouse_id, sprite_group_id, initial_as, location, tap_id, immortal=True):
-        mouse = Mouse(mouse_id, tap_id, initial_as, location, immortal)
+        mouse = animal.Mouse(mouse_id, tap_id, initial_as, location, immortal)
         mouse.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(mouse)
         self.animals.append(mouse)
@@ -144,7 +181,7 @@ class Cast:
         return mouse
 
     def add_frog(self, frog_id, sprite_group_id, initial_as, location, tap_id):
-        frog = Frog(frog_id, tap_id, initial_as, location)
+        frog = animal.Frog(frog_id, tap_id, initial_as, location)
         frog.set_animatory_states(*self.sprite_groups[sprite_group_id])
         self.everything.append(frog)
         self.animals.append(frog)
@@ -166,10 +203,10 @@ class Cast:
         return plant
 
     def get_animatory_state(self, state):
-        width = self.sprites.get_width() // 16
-        height = self.sprites.get_height() // 8
-        surface = self.sprites.subsurface((width * (state % 16), height * (state // 16)), (width, height))
-        surface.set_colorkey((0, 254, 0))
+        width = self.sprites.get_width() // SPRITE_COLUMNS
+        height = self.sprites.get_height() // SPRITE_ROWS
+        surface = self.sprites.subsurface((width * (state % SPRITE_COLUMNS), height * (state // SPRITE_COLUMNS)), (width, height))
+        surface.set_colorkey(SPRITE_COLORKEY)
         return surface
 
     def initialise(self, skool, screen, beeper):
@@ -214,10 +251,12 @@ class Cast:
         self.taps[tap_id].add_command(command_class, *params)
 
     def set_random_locations(self, character_id, locations):
-        self.characters[character_id].set_random_locations(locations)
+        if character_id in self.characters:
+            self.characters[character_id].set_random_locations(locations)
 
     def add_command_list(self, character_id, lesson_id, tap_id):
-        self.characters[character_id].add_command_list(lesson_id, tap_id)
+        if character_id in self.characters:
+            self.characters[character_id].add_command_list(lesson_id, tap_id)
 
     def set_location(self, character_id, x, y):
         self.get(character_id).set_initial_location(x, y)
@@ -232,7 +271,7 @@ class Cast:
         self.characters[character_id].add_blackboard_message(message)
 
     def add_lines_message(self, character_id, message_id, message):
-        message_lines = message.split('^')
+        message_lines = message.split(LINES_MESSAGE_NEWLINE)
         if character_id == '*':
             for c in self.get_lines_givers():
                 c.add_lines_message(message_id, message_lines)
@@ -280,14 +319,14 @@ class Cast:
         return any(c.can_kiss_eric() for c in self.character_list)
 
     def kissee(self):
-        for c in self.get_facing_characters(self.eric, 2):
+        for c in self.get_facing_characters(self.eric, KISS_DISTANCE):
             if c.can_kiss_eric() and c.is_interruptible():
                 return c
 
     def get_punchees(self, character, offset):
         punchees = []
         for c in self.get_facing_characters(character, offset):
-            if c.is_punchable():
+            if c.is_punchable_now():
                 punchees.append(c)
         return punchees
 
@@ -328,16 +367,14 @@ class Cast:
         return [c for c in self.character_list if c.can_give_lines()]
 
     def get_nearby_characters(self, character, candidates, witness):
-        """Returns a list of characters from the given list of candidates who
-        are close enough to the given character to be visible to him
-        (regardless of the direction he's facing)."""
-        x0 = character.x - 10
-        x1 = character.x + 10
-        y0 = character.y - 3
-        y1 = character.y + 3
+        """Return a list of characters from the given list of candidates who are close enough to the given character to be visible to him (regardless of the direction he's facing)."""
+        x0 = character.x - NEARBY_X_RANGE
+        x1 = character.x + NEARBY_X_RANGE
+        y0 = character.y - NEARBY_Y_RANGE
+        y1 = character.y + NEARBY_Y_RANGE
         nearby_characters = []
         for c in candidates:
-            if x0 <= c.x <= x1 and y0 <= c.y <= y1 and c.has_line_of_sight_to(character):
+            if c is not character and x0 <= c.x <= x1 and y0 <= c.y <= y1 and c.has_line_of_sight_to(character):
                 if not witness or c.direction * (character.x - c.x) >= 0:
                     nearby_characters.append(c)
         return nearby_characters
@@ -382,12 +419,18 @@ class Cast:
             if animal.y == character.y and animal.x == x and animal.is_sitting():
                 return animal
 
+    def is_bike_visible(self):
+        return self.bike and self.bike.is_visible()
+
     def is_beside_bike(self, character):
-        return (self.bike.x, self.bike.y) == (character.x, character.y)
+        return self.bike and (self.bike.x, self.bike.y) == (character.x, character.y)
+
+    def move_bike_away(self, door):
+        if self.bike and door.top_y <= self.bike.y <= door.bottom_y and door.x - 2 <= self.bike.x <= door.x:
+            self.bike.x = door.x + 1
 
     def expand_names(self, message):
-        """Replace occurrences of $BLAH with the name of the character whose
-        unique ID is 'BLAH'."""
+        """Replace occurrences of $BLAH with the name of the character whose unique ID is 'BLAH'."""
         index = 0
         marker = '$'
         while message.find(marker, index) >=0:
@@ -402,38 +445,49 @@ class Cast:
         return message
 
     def is_home(self, x):
-        """Returns whether every character is on the 'home' side of the given
-        x-coordinate."""
+        """Return whether every character is on the 'home' side of the given x-coordinate."""
         return all(c.is_home(x) for c in self.character_list)
 
     def is_standing_on_kid(self, character):
-        """Returns whether the character is standing on a kid who's been knocked
-        out."""
+        """Return whether the character is standing on a kid who's been knocked out."""
         for c in self.character_list:
             if c.is_knocked_out() and c.x - 1 <= character.x <= c.x + 1 and character.y == c.y - 1:
                 return True
 
     def somebody_near_door(self, door):
-        """Returns True if somebody is standing near the door, False
-        otherwise."""
+        """Return True if somebody is standing near the door, False otherwise."""
         for c in self.character_list:
             if door.x - 2 <= c.x <= door.x + 1 and door.top_y <= c.y <= door.bottom_y:
                 return True
 
     def is_beside_eric(self, character):
-        """Returns whether the character is beside Eric (and so need go no
-        further to find him)."""
+        """Return whether the character is beside Eric (and so need go no further to find him)."""
         eric_x, eric_y = self.get_location_of_eric()
-        return abs(character.x - eric_x) <= 4 and character.y == eric_y
+        return abs(character.x - eric_x) <= BESIDE_ERIC_X_RANGE and character.y == eric_y
 
     def get_location_of_eric(self):
-        """Returns the non-staircase location closest to Eric."""
+        """Return the non-staircase location closest to Eric."""
         return self.eric.get_location()
 
-    def expel_eric(self):
+    def is_eric_expelled(self):
+        return self.eric.expelled
+
+    def expel_eric(self, command_list_id):
         for c in self.character_list:
             if c.can_expel_eric():
-                c.find_and_expel_eric()
+                self.eric.expelled = True
+                self.change_command_list(c, command_list_id)
+                return
+
+    def get_eric_stopper(self):
+        for c in self.character_list:
+            if c.is_stopping_eric(self.eric):
+                return c
+
+    def shadow_eric(self, command_list_id):
+        for c in self.character_list:
+            if c.can_shadow_eric():
+                self.change_command_list(c, command_list_id)
                 return
 
     def freeze_eric(self):
@@ -454,14 +508,14 @@ class Cast:
     def open_desk(self, character, desk):
         self.desk_lid.raise_lid(desk, character)
 
-    def trip_people_up_at(self, x, y):
+    def trip_people_up_at(self, character, x, y):
         # Assume trippability == pelletability
         for trippable in self.get_pelletables():
-            if (trippable.x, trippable.y) == (x, y) and trippable.is_deckable():
+            if trippable is not character and (trippable.x, trippable.y) == (x, y) and trippable.is_deckable():
                 trippable.deck()
 
     def water_plant(self, plant_pot, liquid):
-        if liquid == 'WATER':
+        if liquid == char.WATER:
             plant_pot.plant.grow()
 
     def lift_anyone_at(self, x, y):
@@ -484,10 +538,10 @@ class Cast:
             for frog in cup.frogs:
                 frog.fall_from_cup(cup)
             return
-        x, y = cup.x - 1, cup.y - 3
-        if cup.contents == 'WATER' and not self.water_drop.is_visible():
+        x, y = cup.x - droppable.DROP_X, cup.y - droppable.DROP_Y
+        if cup.contents == char.WATER and not self.water_drop.is_visible():
             self.water_drop.fall(x, y)
-        elif cup.contents == 'SHERRY' and not self.sherry_drop.is_visible():
+        elif cup.contents == char.SHERRY and not self.sherry_drop.is_visible():
             self.sherry_drop.fall(x, y)
 
     def hit_conker(self, pellet):
@@ -502,10 +556,6 @@ class Cast:
     def unchain_bike(self):
         self.bike.unchain()
 
-    def release_frogs(self):
-        for frog in self.frogs:
-            frog.release()
-
     def insert_frog(self, cup):
         for frog in self.frogs:
             if not frog.is_visible():
@@ -515,14 +565,17 @@ class Cast:
     def check_heads(self, frog):
         for c in self.character_list:
             if c.has_safe_key():
-                got_key = False
+                hit_head = False
+                x_delta = animal.FROG_X - 1
+                y_delta = animal.FROG_Y + 1
                 if c.is_standing():
-                    got_key = (frog.x, frog.y) == (c.x, c.y - 4)
+                    hit_head = (frog.x, frog.y) == (c.x - x_delta, c.y - y_delta)
                 elif c.is_knocked_over():
-                    got_key = (frog.x, frog.y) == (c.x, c.y - 3)
-                if got_key:
+                    hit_head = (frog.x, frog.y) == (c.x - x_delta, c.y - y_delta + 1)
+                if hit_head:
                     self.eric.take_safe_key()
-                    break
+                    return True
+        return False
 
     def smeller(self, stinkbomb):
         for c in self.character_list:
@@ -536,7 +589,7 @@ class Cast:
             self.kill_mouse(mouse)
 
     def relocate_mouse(self, mouse):
-        locations = [(10, 3), (88, 3), (170, 3), (10, 10), (88, 10), (170, 10), (10, 17), (88, 17), (170, 17)]
+        locations = list(MOUSE_LOCATIONS)
         while locations and self.screen.contains(mouse):
             mouse.x, mouse.y = locations.pop(random.randrange(len(locations)))
         mouse.restart_table()
@@ -544,8 +597,8 @@ class Cast:
     def release_mice(self, num_mice, x, y):
         while num_mice > 0:
             mouse_id = 'MortalMouse%i' % num_mice
-            tap_id = 'Mouse'
-            mouse = self.add_mouse(mouse_id, 'MOUSE', RUN, (x, y), tap_id, False)
+            tap_id = MOUSE_COMMAND_LIST
+            mouse = self.add_mouse(mouse_id, MOUSE_SPRITE_GROUP, RUN, (x, y), tap_id, False)
             self.initialise_character(mouse)
             mouse.set_command_list_template(self.taps[tap_id])
             num_mice -= 1
@@ -559,6 +612,9 @@ class Cast:
         for c in self.character_list:
             c.check_mouse(mouse)
 
+    def change_command_list(self, character, command_list_id):
+        character.set_command_list_template(self.taps[command_list_id])
+
     ############################################################################
     # Grass config
     ############################################################################
@@ -578,86 +634,19 @@ class Cast:
         self.absent_tale = absent_tale
 
     def expand_title(self, message, character):
-        return message.replace('$TITLE', character.get_title())
+        return message.replace(TITLE_MARKER, character.get_title())
 
     def get_hit_tale(self, teacher):
         hitter_id = random.choice(self.hitters)
-        message = self.expand_names(self.hit_tale.replace('$1', '$%s' % hitter_id))
+        message = self.expand_names(self.hit_tale.replace(GRASSEE_MARKER, '$%s' % hitter_id))
         return hitter_id, self.expand_title(message, teacher)
 
     def get_write_tale(self, writer_id, teacher):
         if writer_id in self.writers:
             culprit = random.choice(self.writers)
-            message = self.expand_names(self.write_tale.replace('$1', '$%s' % writer_id))
+            message = self.expand_names(self.write_tale.replace(GRASSEE_MARKER, '$%s' % writer_id))
             return writer_id, self.expand_title(message, teacher)
         return None, None
 
     def get_absent_tale(self, teacher):
         return self.expand_title(self.expand_names(self.absent_tale), teacher)
-
-class Droppable(Character):
-    def __init__(self, object_id, tap_id, animatory_state, direction):
-        Character.__init__(self, object_id, object_id)
-        self.tap_id = tap_id
-        self.animatory_state = self.initial_as = animatory_state
-        self.direction = self.initial_direction = direction
-        self.x = -3
-        self.y = 0
-        self.initial_location = Location((self.x, self.y))
-
-    def get_tap_id(self, lesson_id):
-        return self.tap_id
-
-    def fall(self, x, y):
-        self.x = x
-        self.y = y
-
-class WaterDrop(Droppable):
-    def __init__(self, object_id, tap_id):
-        Droppable.__init__(self, object_id, tap_id, WATER_DROP, -1)
-
-    def hit_victim(self):
-        victim = self.cast.get_waterable(self.x, self.y + 4)
-        if victim and victim.is_standing():
-            victim.reveal_bike_secret()
-            victim.deck()
-            return True
-        victim = self.cast.get_waterable(self.x, self.y + 3)
-        if victim and victim.is_knocked_over():
-            victim.reveal_bike_secret()
-            return True
-        return False
-
-class SherryDrop(Droppable):
-    def __init__(self, object_id, tap_id):
-        Droppable.__init__(self, object_id, tap_id, SHERRY_DROP, 1)
-
-    def hit_victim(self):
-        victim = self.cast.get_sherryable(self.x, self.y + 4)
-        if victim and victim.is_standing():
-            victim.reveal_storeroom_secret()
-            victim.deck()
-            return True
-        victim = self.cast.get_sherryable(self.x, self.y + 3)
-        if victim and victim.is_knocked_over():
-            victim.reveal_storeroom_secret()
-            return True
-        return False
-
-class Conker(Droppable):
-    def __init__(self, object_id, tap_id, min_x, max_x, min_y, max_y):
-        Droppable.__init__(self, object_id, tap_id, CONKER, -1)
-        self.min_x = min_x
-        self.max_x = max_x
-        self.min_y = min_y
-        self.max_y = max_y
-
-    def hit_by(self, pellet):
-        return self.min_x <= pellet.x <= self.max_x and self.min_y <= pellet.y <= self.max_y
-
-    def hit_victim(self):
-        victim = self.cast.get_conkerable(self.x, self.y + 4)
-        if victim:
-            victim.deck(victim.is_very_conkerable())
-            return True
-        return False
