@@ -15,27 +15,52 @@
 # You should have received a copy of the GNU General Public License along with
 # Pyskool. If not, see <http://www.gnu.org/licenses/>.
 
-from character import Character
+import character
 from animatorystates import *
-from ai import Write, Jump, Hit, FireCatapult, Freeze, Catch, FireWaterPistol, DumpWaterPistol, DropStinkbomb, RideBike, FallToFloor, JumpOutOfWindow, ClimbOverSkoolGate, Kiss, ReleaseMice
+from ai import Write, Jump, Hit, FireCatapult, Freeze, Catch, FireWaterPistol, DumpWaterPistol, DropStinkbomb, RideBike, FallToFloor, JumpOutOfWindow, ClimbOverSkoolGate, Kiss, ReleaseMice, SKOOL_GATE
 import lines
 import items
 import bike
 import keys
 
+# Minimum delay between two non-immediate lines-givings by the same teacher
 LINES_DELAY = 60
 
 # The command list Mr Wacker should use to expel Eric after he's jumped out of
 # the top-floor window
 NOT_A_BIRD = 'ExpelEric:NotABird'
 
-class Eric(Character):
+# ID of the science lab storeroom door
+STOREROOM_DOOR = 'ScienceLabDoor'
+
+# ID of the assembly hall
+ASSEMBLY_HALL = 'AssemblyHall'
+
+# Delay before Eric returns to the floor after jumping
+JUMP_DELAY = 4
+# Delay before Eric stands up after bending over
+BEND_OVER_DELAY = 4
+
+# Maximum number of mice to release per attempt
+MAX_MICE_RELEASE = 5
+
+# Points scored when bike is unchained
+BIKE_COMBO_SCORE = 1000
+# Points scored when storeroom combination is obtained
+STOREROOM_COMBO_SCORE = 1000
+# Points scored when safe key is obtained
+SAFE_KEY_SCORE = 2000
+
+# Number of lines kissee does for Eric on each kiss
+KISS_LINES = 1000
+
+class Eric(character.Character):
     def __init__(self, character_id, name, flags):
-        Character.__init__(self, character_id, name, flags)
+        character.Character.__init__(self, character_id, name, flags)
         self._reinitialise()
 
     def reinitialise(self):
-        Character.reinitialise(self)
+        character.Character.reinitialise(self)
         self._reinitialise()
 
     def _reinitialise(self):
@@ -137,7 +162,7 @@ class Eric(Character):
         if self.keyboard.pressed(*keys.LEFT):
             if plant and plant.is_fully_grown():
                 barrier = self.skool.barrier(self, 1)
-                if barrier and barrier.barrier_id == 'SkoolGate' and barrier.is_shut():
+                if barrier and barrier.barrier_id == SKOOL_GATE and barrier.is_shut():
                     self.controller = ClimbOverSkoolGate()
                     return 0
             self.left()
@@ -147,7 +172,7 @@ class Eric(Character):
                 if window and not window.is_shut():
                     self.controller = JumpOutOfWindow(window.barrier_id)
                     return 0
-                elif self.barrier and self.barrier.barrier_id == 'SkoolGate' and self.barrier.is_shut():
+                elif self.barrier and self.barrier.barrier_id == SKOOL_GATE and self.barrier.is_shut():
                     self.controller = ClimbOverSkoolGate()
                     return 0
             self.right()
@@ -160,7 +185,7 @@ class Eric(Character):
         return 0
 
     def can_fire_catapult(self):
-        return Character.can_fire_catapult(self) and not self.cast.conker_falling()
+        return character.Character.can_fire_catapult(self) and not self.cast.conker_falling()
 
     def can_sit_on_stairs(self):
         """Return whether Eric can sit on the stairs."""
@@ -175,9 +200,9 @@ class Eric(Character):
             or self.cast.plant(self))
 
     def can_open_door(self, door):
-        if door.barrier_id == 'ScienceLabDoor' and items.STOREROOM_KEY in self.inventory:
+        if door.barrier_id == STOREROOM_DOOR and items.STOREROOM_KEY in self.inventory:
             return True
-        return Character.can_open_door(self, door)
+        return character.Character.can_open_door(self, door)
 
     def open_door(self):
         self.skool.move_door(self.barrier.barrier_id, False)
@@ -205,10 +230,9 @@ class Eric(Character):
             self.alert_lines_givers(lines.GET_OFF_PLANT, True)
         else:
             is_assembly = self.skool.is_assembly()
-            assembly_hall_id = 'AssemblyHall'
             if self.is_sitting_on_floor():
                 # Eric is sitting on the floor...
-                if room and room.room_id == assembly_hall_id:
+                if room and room.room_id == ASSEMBLY_HALL:
                     if self.direction < 0:
                         # ...facing left in the assembly hall...
                         if is_assembly:
@@ -223,7 +247,7 @@ class Eric(Character):
                     # ...somewhere other than the assembly hall - tell him to
                     # get off the floor
                     self.alert_lines_givers(lines.GET_UP, True)
-            elif self.is_standing() and home_room and home_room.contains(self) and (home_room.chairs or home_room.room_id == assembly_hall_id):
+            elif self.is_standing() and home_room and home_room.contains(self) and (home_room.chairs or home_room.room_id == ASSEMBLY_HALL):
                 # Eric is standing up either in a classroom after the lesson
                 # has started, or in the assembly hall after assembly has
                 # started - tell him to sit down
@@ -241,7 +265,7 @@ class Eric(Character):
         self.beeper.make_walking_sound(index)
 
     def walk(self, on_stairs=False):
-        Character.walk(self, on_stairs)
+        character.Character.walk(self, on_stairs)
         if not self.midstride():
             return self.screen.get_scroll_increment(self.x)
         return 0
@@ -279,7 +303,7 @@ class Eric(Character):
             self.last_lines_giver = lines_giver
 
     def aim_catapult(self):
-        Character.aim_catapult(self)
+        character.Character.aim_catapult(self)
         self.alert_lines_givers(lines.NO_CATAPULTS)
         self.beeper.make_catapult_sound()
 
@@ -288,7 +312,7 @@ class Eric(Character):
 
     def can_fire_water_pistol(self):
         if self.has_water_pistol():
-            return Character.can_fire_water_pistol(self)
+            return character.Character.can_fire_water_pistol(self)
 
     def can_dump_water_pistol(self):
         return self.has_water_pistol() and self.can_bend_over()
@@ -303,21 +327,22 @@ class Eric(Character):
             self.inventory.remove(pistol)
             self.print_inventory()
             self.skool.hide_in_desk(items.WATER_PISTOL)
+        self.walk_delay = BEND_OVER_DELAY
 
     def fire_water_pistol(self):
-        liquid = 'SHERRY' if items.SHERRY_PISTOL in self.inventory else 'WATER'
-        Character.fire_water_pistol(self, liquid)
+        liquid = character.SHERRY if items.SHERRY_PISTOL in self.inventory else character.WATER
+        character.Character.fire_water_pistol(self, liquid)
         self.alert_lines_givers(lines.NO_WATERPISTOLS)
         self.beeper.make_catapult_sound()
 
     def can_drop_stinkbomb(self):
-        return self.floor and self.has_stinkbomb() and Character.can_drop_stinkbomb(self)
+        return self.floor and self.has_stinkbomb() and character.Character.can_drop_stinkbomb(self)
 
     def has_stinkbomb(self):
         return any([i in self.inventory for i in (items.STINKBOMBS1, items.STINKBOMBS2, items.STINKBOMBS3)])
 
     def drop_stinkbomb(self):
-        Character.drop_stinkbomb(self)
+        character.Character.drop_stinkbomb(self)
         if items.STINKBOMBS3 in self.inventory:
             self.inventory.remove(items.STINKBOMBS3)
             self.inventory.add(items.STINKBOMBS2)
@@ -346,13 +371,13 @@ class Eric(Character):
             self.print_inventory()
 
     def punch(self):
-        Character.punch(self)
+        character.Character.punch(self)
         self.alert_lines_givers(lines.NO_HITTING)
 
     def jump(self):
         self.previous_as = self.animatory_state
         self.y -= 1
-        self.walk_delay = 4
+        self.walk_delay = JUMP_DELAY
         self.animatory_state = ARM_UP
         self.alert_lines_givers(lines.NO_JUMPING)
 
@@ -378,7 +403,7 @@ class Eric(Character):
                 self.beeper.make_bingo_sound()
                 self.inventory.add(items.FROG)
                 self.print_inventory()
-        self.walk_delay = 4
+        self.walk_delay = BEND_OVER_DELAY
 
     def print_mouse_inventory(self):
         self.screen.print_mice(self.mice, self.skool.get_mouse_image())
@@ -388,10 +413,11 @@ class Eric(Character):
 
     def release_mice(self):
         if self.mice > 0:
-            num_mice = min(self.mice, 5)
+            num_mice = min(self.mice, MAX_MICE_RELEASE)
             self.mice -= num_mice
             self.print_mouse_inventory()
             self.cast.release_mice(num_mice, self.x + self.direction, self.y)
+        self.walk_delay = BEND_OVER_DELAY
 
     def print_inventory(self):
         self.screen.print_inventory(self.skool.get_inventory_images(self.inventory))
@@ -406,12 +432,7 @@ class Eric(Character):
             # If Eric is hiding (e.g. while kissing), temporarily restore his
             # pre-hide coordinates for the location check
             self.x, self.y = self.hide_coords
-        staircase = self.skool.staircase(self)
-        if staircase and staircase.supports(self):
-            if staircase.bottom.y - self.y < self.y - staircase.top.y:
-                return staircase.bottom.coords()
-            return staircase.top.coords()
-        location = (self.x, self.skool.floor_below(self).y)
+        location = character.Character.get_location(self)
         if self.hide_coords:
             self.hide()
         return location
@@ -434,9 +455,8 @@ class Eric(Character):
         self.animatory_state = self.previous_as
 
     def write(self):
-        """Controls Eric when he's writing on a board. Returns True if Eric has
-        finished writing."""
-        blackboard = self.skool.room(self).get_blackboard()
+        """Control Eric when he's writing on a board. Return True if Eric has finished writing."""
+        blackboard = self.skool.room(self).blackboard
         if self.has_arm_raised():
             self.lower_arm()
             self.writing = self.keyboard.writing
@@ -457,15 +477,16 @@ class Eric(Character):
                 self.raise_arm()
                 self.skool.write_on_board(self, blackboard, char)
         else:
-            self.walk_delay = 1 # Maximise responsiveness (hack)
+             # Maximise responsiveness
+            self.walk_delay = 1
 
     def check_combinations(self, blackboard):
-        if self.skool.got_bike_combination(blackboard):
-            self.skool.add_to_score(100)
+        if not self.cast.is_bike_visible() and self.skool.got_bike_combination(blackboard):
+            self.skool.add_to_score(BIKE_COMBO_SCORE // 10)
             self.skool.unchain_bike()
             self.beeper.make_bingo_sound()
-        elif self.skool.got_storeroom_combination(blackboard):
-            self.skool.add_to_score(100)
+        elif items.STOREROOM_KEY not in self.inventory and self.skool.got_storeroom_combination(blackboard):
+            self.skool.add_to_score(STOREROOM_COMBO_SCORE // 10)
             self.inventory.add(items.STOREROOM_KEY)
             self.print_inventory()
             self.beeper.make_bingo_sound()
@@ -496,10 +517,11 @@ class Eric(Character):
         self.cast.expel_eric(NOT_A_BIRD)
 
     def take_safe_key(self):
-        self.skool.add_to_score(200)
-        self.beeper.make_safe_key_sound()
-        self.inventory.add(items.SAFE_KEY)
-        self.print_inventory()
+        if items.SAFE_KEY not in self.inventory:
+            self.skool.add_to_score(SAFE_KEY_SCORE // 10)
+            self.beeper.make_safe_key_sound()
+            self.inventory.add(items.SAFE_KEY)
+            self.print_inventory()
 
     def can_kiss(self):
         return self.cast.has_kissees()
@@ -508,12 +530,12 @@ class Eric(Character):
         return self.cast.kissee()
 
     def kiss(self):
-        self.skool.add_lines(-100)
+        self.skool.add_lines(-KISS_LINES // 10)
         self.beeper.make_bingo_sound()
 
     def hide(self):
         self.hide_coords = (self.x, self.y)
-        Character.hide(self)
+        character.Character.hide(self)
 
     def unhide(self):
         if self.hide_coords:
