@@ -18,6 +18,64 @@
 import os
 import pygame
 
+# Width of the screen (in tiles)
+WIDTH = 32
+# Height of the play area (in tiles)
+SKOOL_HEIGHT = 21
+# Height of the screen (in tiles)
+HEIGHT = SKOOL_HEIGHT + 3
+
+# x-coordinate of Eric at which screen will be scrolled right
+SCROLL_MIN_X = 9
+# x-coordinate of Eric at which screen will be scrolled left
+SCROLL_MAX_X = WIDTH - 10
+# Number of columns to scroll
+SCROLL_COLS = 8
+
+# Size of the lesson box (width and height in tiles)
+LESSON_BOX_SIZE = (8, 3)
+# Size of the inventory (width and height in tiles)
+INVENTORY_SIZE = (6, 1)
+# Size of a lines bubble (width and height in tiles)
+LINES_BUBBLE_SIZE = (8, 3)
+
+# Offsets used to position printing of score, lines total and hi-score
+SCORE_Y_OFFSET = 1
+LINES_Y_OFFSET = 9
+HISCORE_Y_OFFSET = 17
+
+# Colours
+BLACK = (0, 0, 0)
+BLUE = (0, 0, 197)
+RED = (197, 0, 0)
+MAGENTA = (197, 0, 197)
+GREEN = (0, 198, 0)
+CYAN = (0, 198, 197)
+YELLOW = (197, 198, 0)
+WHITE = (205, 198, 205)
+BRIGHT_BLACK = (0, 0, 0)
+BRIGHT_BLUE = (0, 0, 255)
+BRIGHT_RED = (255, 0, 0)
+BRIGHT_MAGENTA = (255, 0, 255)
+BRIGHT_GREEN = (0, 255, 0)
+BRIGHT_CYAN = (0, 255, 255)
+BRIGHT_YELLOW = (255, 255, 0)
+BRIGHT_WHITE = (255, 255, 255)
+
+# Size of the bounding rectangle of a speech bubble (width and height in tiles)
+SPEECH_BUBBLE_SIZE = (8, 3)
+# Transparent colour used in the speech bubble
+SPEECH_BUBBLE_COLORKEY = (0, 255, 0)
+# Bit pattern used to open the lip of the speech bubble
+OPEN_LIP_BYTE = 66 # 01000010
+
+# Transparent colour used in the skool ink image
+SKOOL_COLORKEY = (255, 255, 255)
+
+# Ink and paper colours in font.png (used to create transparency)
+FONT_INK = (0, 1, 2)
+FONT_PAPER = (255, 254, 253)
+
 class Screen:
     def __init__(self, mode, scale, background, gallery, scroll_fps, title, icon_fname):
         self.initialise_colours()
@@ -25,7 +83,7 @@ class Screen:
         self.scale = scale
         self.background = background
         self.scroll_fps = scroll_fps
-        self.screen = pygame.display.set_mode((256 * self.scale, 192 * self.scale))
+        self.screen = pygame.display.set_mode(self.scale_coords((WIDTH, HEIGHT)))
         if os.path.isfile(icon_fname):
             pygame.display.set_icon(pygame.image.load(icon_fname).convert())
         pygame.display.set_caption(title)
@@ -42,27 +100,27 @@ class Screen:
 
     def initialise_colours(self):
         self.colours = []
-        self.colours.append((0, 0, 0))       #  0 - black
-        self.colours.append((0, 0, 197))     #  1 - blue
-        self.colours.append((197, 0, 0))     #  2 - red
-        self.colours.append((197, 0, 197))   #  3 - magenta
-        self.colours.append((0, 198, 0))     #  4 - green
-        self.colours.append((0, 198, 197))   #  5 - cyan
-        self.colours.append((197, 198, 0))   #  6 - yellow
-        self.colours.append((205, 198, 205)) #  7 - white
-        self.colours.append((0, 0, 0))       #  8 - bright black
-        self.colours.append((0, 0, 255))     #  9 - bright blue
-        self.colours.append((255, 0, 0))     # 10 - bright red
-        self.colours.append((255, 0, 255))   # 11 - bright magenta
-        self.colours.append((0, 255, 0))     # 12 - bright green
-        self.colours.append((0, 255, 255))   # 13 - bright cyan
-        self.colours.append((255, 255, 0))   # 14 - bright yellow
-        self.colours.append((255, 255, 255)) # 15 - bright white
+        self.colours.append(BLACK)
+        self.colours.append(BLUE)
+        self.colours.append(RED)
+        self.colours.append(MAGENTA)
+        self.colours.append(GREEN)
+        self.colours.append(CYAN)
+        self.colours.append(YELLOW)
+        self.colours.append(WHITE)
+        self.colours.append(BRIGHT_BLACK)
+        self.colours.append(BRIGHT_BLUE)
+        self.colours.append(BRIGHT_RED)
+        self.colours.append(BRIGHT_MAGENTA)
+        self.colours.append(BRIGHT_GREEN)
+        self.colours.append(BRIGHT_CYAN)
+        self.colours.append(BRIGHT_YELLOW)
+        self.colours.append(BRIGHT_WHITE)
 
     def initialise_column(self, initial_column, skool):
-        self.max_column = skool.get_width() - 32
+        self.max_column = skool.get_width() - WIDTH
         eric_x = skool.get_eric().x
-        default_initial_column = min(self.max_column, max((eric_x - 16) // 8 * 8, 0))
+        default_initial_column = min(self.max_column, max((eric_x - WIDTH // 2) // 8 * 8, 0))
         column = min(self.max_column, default_initial_column if initial_column is None else initial_column)
         self.column = self.initial_column = max(0, 8 * (column // 8))
 
@@ -93,16 +151,16 @@ class Screen:
 
     def get_scroll_increment(self, x):
         offset = x - self.column
-        if 21 < offset < 32:
+        if SCROLL_MAX_X <= offset < WIDTH:
             return 1
-        if -3 < offset < 10:
+        if -3 < offset <= SCROLL_MIN_X:
             return -1
         return 0
 
     def scroll_skool(self, skool, clock):
-        self.column -= 32
+        self.column -= WIDTH
         background = pygame.Surface((self.screen.get_width(), 8 * self.scale * skool.get_height()))
-        for n in range(32):
+        for n in range(WIDTH):
             self.column += 1
             skool.draw()
             self.screen.blit(background, (-8 * (n + 1) * self.scale, 0))
@@ -114,7 +172,7 @@ class Screen:
             return
         if self.column == 0 and inc < 0:
             return
-        for i in range(0, 8):
+        for i in range(SCROLL_COLS):
             self.column += inc
             skool.draw()
             self.update()
@@ -129,7 +187,7 @@ class Screen:
         return ink, paper
 
     def print_lesson(self, teacher, room):
-        lesson_box = pygame.Surface((self.scale * 64, self.scale * 24))
+        lesson_box = pygame.Surface(self.scale_coords(LESSON_BOX_SIZE))
         ink, paper = self.get_ink_and_paper(self.lesson_box_attr)
         lesson_box.fill(paper)
         teacher_text = self.get_text(teacher, ink, paper)
@@ -160,18 +218,18 @@ class Screen:
         self.update()
 
     def print_score(self, score):
-        self.print_number(score, 1)
+        self.print_number(score, SCORE_Y_OFFSET)
 
     def print_lines(self, lines):
-        self.print_number(lines, 9)
+        self.print_number(lines, LINES_Y_OFFSET)
 
     def print_hi_score(self, hiscore):
-        self.print_number(hiscore, 17)
+        self.print_number(hiscore, HISCORE_Y_OFFSET)
 
     def print_inventory(self, item_images=()):
         if not self.inventory_coords:
             return
-        inventory_box = pygame.Surface((self.scale * 48, self.scale * 8))
+        inventory_box = pygame.Surface(self.scale_coords(INVENTORY_SIZE))
         ink, paper = self.get_ink_and_paper(self.background)
         inventory_box.fill(paper)
         x = 0
@@ -196,13 +254,13 @@ class Screen:
 
     def contains(self, character, full=True):
         """Return whether a character is on-screen."""
-        return self.column <= character.x <= self.column + (29 if full else 31)
+        return self.column <= character.x <= self.column + WIDTH - (3 if full else 1)
 
     def print_lines_bubble(self, x, y, message, ink_colour, paper_colour):
         bubble_x = 8 * (x // 8) - self.column
         bubble_coords = self.scale_coords((bubble_x, y))
 
-        lines_bubble = pygame.Surface(self.scale_coords((8, 3)))
+        lines_bubble = pygame.Surface(self.scale_coords(LINES_BUBBLE_SIZE))
         ink = self.colours[ink_colour]
         paper = self.colours[paper_colour]
         lines_bubble.fill(paper)
@@ -220,15 +278,14 @@ class Screen:
     def get_bubble(self, words, lip_pos, shift):
         ink = self.colours[0]
         paper = self.colours[7]
-        bubble = pygame.Surface(self.scale_coords((8, 3)))
-        colorkey = (0, 255, 0)
-        bubble.fill(colorkey)
-        bubble.set_colorkey(colorkey)
+        bubble = pygame.Surface(self.scale_coords(SPEECH_BUBBLE_SIZE))
+        bubble.fill(SPEECH_BUBBLE_COLORKEY)
+        bubble.set_colorkey(SPEECH_BUBBLE_COLORKEY)
         bubble.blit(self.bubble, (0, 0))
         lip = self.bubble.subsurface(self.scale_coords((8, 0)), self.scale_coords((1, 1)))
         lip_coords = self.scale_coords((lip_pos, 2))
         bubble.blit(lip, lip_coords)
-        open_lip_byte = 66
+        open_lip_byte = OPEN_LIP_BYTE
         for bit in range(0, 8 * self.scale, self.scale):
             colour = ink if open_lip_byte & 128 else paper
             for r in range(0, self.scale + 1):
@@ -239,9 +296,10 @@ class Screen:
         inset_x = (4 - 8 * min(shift, 0)) * self.scale
         inset_y = 4 * self.scale
         text_x = max(shift, 0) * 8 * self.scale
-        width = min(56 * self.scale, text.get_width() - text_x)
-        if inset_x + width > 56 * self.scale:
-            width = 56 * self.scale - inset_x
+        max_width = self.scale * 8 * (SPEECH_BUBBLE_SIZE[0] - 1)
+        width = min(max_width, text.get_width() - text_x)
+        if inset_x + width > max_width:
+            width = max_width - inset_x
         text_window = text.subsurface((text_x, 0), (width, 8 * self.scale))
         bubble.blit(text_window, (inset_x, inset_y))
         return (bubble, text_x > text.get_width())
@@ -257,14 +315,14 @@ class Screen:
             for x, y, sprite in cast:
                 self.draw_image(self.screen, x, y, sprite)
         else:
-            scratch = pygame.Surface(self.scale_coords((32, 21)))
+            scratch = pygame.Surface(self.scale_coords((WIDTH, SKOOL_HEIGHT)))
             self.draw_skool(scratch, skool_images[1])
             for x, y, text in blackboards:
                 self.draw_blackboard(scratch, x, y, text)
             for x, y, sprite in cast:
                 self.draw_image(scratch, x, y, sprite)
             self.draw_skool(self.screen, skool_images[2])
-            scratch.set_colorkey((255, 255, 255))
+            scratch.set_colorkey(SKOOL_COLORKEY)
             self.screen.blit(scratch, (0, 0))
         for x, y, image in others:
             self.draw_image(self.screen, x, y, image)
@@ -358,15 +416,13 @@ class Font:
         for image in character_images:
             text.blit(image, (offset, 0))
             offset += image.get_width()
-        text_ink = (0, 1, 2)
-        text_paper = (255, 254, 253)
         paper_surface = pygame.Surface((text.get_width(), text.get_height()))
         paper_surface.fill(paper)
         ink_surface = pygame.Surface((text.get_width(), text.get_height()))
         ink_surface.fill(ink)
-        text.set_colorkey(text_ink)
+        text.set_colorkey(FONT_INK)
         ink_surface.blit(text, (0, 0))
-        ink_surface.set_colorkey(text_paper)
+        ink_surface.set_colorkey(FONT_PAPER)
         paper_surface.blit(ink_surface, (0, 0))
         return paper_surface
 
