@@ -17,7 +17,10 @@
 
 from character import Character
 from animatorystates import *
-from skool import Location
+from location import Location
+
+# Coordinates of the pellet within its sprite (used for collision detection)
+PELLET_X, PELLET_Y = 1, 1
 
 class Pellet(Character):
     def __init__(self, pellet_id, tap_id, pellet_range, hit_zone):
@@ -33,17 +36,17 @@ class Pellet(Character):
         return bottom_y >= self.y + 1 >= top_y
 
     def get_victim(self):
-        """Returns whoever has been hit by this pellet, if anyone."""
+        """Return whoever has been hit by this pellet, if anyone."""
         return self.cast.get_pelletable(self.x, self.y)
 
     def get_tap_id(self, lesson_id):
         return self.tap_id
 
     def hit_shield(self):
-        return self.check_shields_at(self.x + 1, self.y + 1)
+        return self.check_shields_at(self.x + PELLET_X, self.y + PELLET_Y)
 
     def hit_cup(self):
-        cup = self.skool.cup(self.x + 1, self.y + 1)
+        cup = self.skool.cup(self.x + PELLET_X, self.y + PELLET_Y)
         if cup and not cup.is_empty():
             self.cast.knock_cup(cup)
             return True
